# secret local settings
import os
from mysmile.settings.base import BASE_DIR

DEBUG = True

COMPRESS_ENABLED = False

DEBUG_TOOLBAR_PATCH_SETTINGS = False

SECRET_KEY = '=)1+x0_f%6t&$4gftv-^^w20kr)+1ut8js0@0(gt6cx8z6kxlc'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'mysmile/db/mysmile.sqlite3'),
        # 'USER': '',
        # 'PASSWORD': '',
        # 'HOST': '',
        # 'PORT': '',
    }
}

# uncomment for use MySql
# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.mysql',
#         'NAME': 'mysmile',
#         'USER': 'root',
#         'PASSWORD': '',
#         'HOST': '127.0.0.1',
#         'PORT': '3306',
#     }
# }


ALLOWED_HOSTS = ['127.0.0.1', 'localhost', 'mysmile.dev']

# Enter path to media folder
MEDIA_ROOT = os.path.join(BASE_DIR, 'mysmile/media/')

# Enter path to static folder
STATIC_ROOT = os.path.join(BASE_DIR, 'mysmile/static/')

COMPRESS_ROOT = STATIC_ROOT

ADMINS = (
    ('admin', 'info@mysmile.com.ua'),
)





# secret production settings
import os
from mysmile.settings.base import BASE_DIR

DEBUG = False

COMPRESS_ENABLED = True

DEBUG_TOOLBAR_PATCH_SETTINGS = DEBUG

SECRET_KEY = ''

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'mysmile/db/mysmile.sqlite3'),
        # 'USER': '',
        # 'PASSWORD': '',
        # 'HOST': '',
        # 'PORT': '',
    }
}

# for example: ALLOWED_HOSTS = ['mysite.com']
ALLOWED_HOSTS = ['demo.mysmile.com.ua']

# Enter path to media folder on server.
MEDIA_ROOT = ''

# Enter path to static folder on server.
STATIC_ROOT = ''

COMPRESS_ROOT = STATIC_ROOT


ADMINS = (
    ('admin', 'support@mysmile.com.ua'),
)

MANAGERS = (
    ('admin', 'support@mysmile.com.ua'),
)

EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
SERVER_EMAIL = 'mail@mysite.com'
DEFAULT_FROM_EMAIL = SERVER_EMAIL
EMAIL_SUBJECT_PREFIX = 'something like [MYSITE]'
EMAIL_HOST = 'adress of IMAP server'
EMAIL_HOST_USER = 'mail@mysite.com'
EMAIL_HOST_PASSWORD = 'password to email'

EMAIL_USE_SSL = True
EMAIL_PORT = 465
# or
# EMAIL_USE_TLS = True
# EMAIL_PORT = 587

# This is only used if CommonMiddleware is installed
PREPEND_WWW = False

# This is only used if CommonMiddleware is installed
APPEND_SLASH = True

# To fix the django.fcgi url problem
FORCE_SCRIPT_NAME = ''



from django.forms import ModelForm

from apps.pages.models import Page_translation, Page
from apps.preferences.models import Preferences

class Page_translationInlineForm(ModelForm):
    class Meta:
        model = Page_translation
        exclude = ['updated_at', 'created_at']


class PageForm(ModelForm):

    def __init__(self, *args, **kwargs):
        super(PageForm, self).__init__(*args, **kwargs)
        """ Temporary disable color field for non-classic themes. TODO: fix this in future.
        """
        self.theme = Preferences.objects.filter(key='THEME').values_list('value', flat=True)[0]
        if self.theme=='classic':
            self.fields['color'].widget.attrs = {'class': "color {hash:true,caps:false}"}
        else:
            # set default color
            self.fields['color'].widget.attrs = {'readonly':'readonly',
                                                 'value': self.Meta.model._meta.get_field('color').get_default()}

    class Meta:
        model = Page
        exclude = ['updated_at', 'created_at']

    class Media:
        js = ('vendor/jscolor/jscolor.js',)
        #css = {
            #'all': ('zzzZZZzzz.css',)
        #}




from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.exceptions import ParseError
from rest_framework import status
from rest_framework.exceptions import PermissionDenied

from django.conf import settings

from apps.api.v2.serializers import ContactSerializer, LangsSerializer, ContentSerializer
from apps.preferences.models import Preferences
from apps.pages.models import Page, Page_translation


class ContactView(APIView):
    def get(self, request, format=None):
        serializer = ContactSerializer()
        content = {'data': serializer.data, 'code': status.HTTP_200_OK}
        return Response(content)

class LangsView(APIView):
    def get(self, request, format=None):
        serializer = LangsSerializer()
        content = {'data': serializer.data, 'code': status.HTTP_200_OK}
        return Response(content)

class ContentView(APIView):

    # def dispatch(self, request, *args, **kwargs):
    #     if not settings.MYSMILE_REST_API:
    #         # raise PermissionDenied('Forbidden')
    #         print('MYSMILE_REST_API == ', settings.MYSMILE_REST_API)
    #     return super(ContentView, self).dispatch(request, *args, **kwargs)

    def get(self, request, format=None):
        """
        @TODO in-progress
        """
        # if not settings.MYSMILE_REST_API:
        #     raise PermissionDenied('Forbidden')

        lang = request.GET.get('lang', 'en')
        slug = request.GET.get('slug', None)


        content = Page_translation.objects.filter(lang=lang,
                                                  page__status=Page.STATUS_PUBLISHED,
                                                  page__ptype__in=[Page.PTYPE_API,
                                                                   Page.PTYPE_MENU_API]
                                                  ).order_by('page__sortorder')


        serializer = ContentSerializer(content, many=True)
        # print('serializer -----get_initial------- ', serializer.get_initial())
        content = {'data': serializer.data, 'code': status.HTTP_200_OK}
        return Response(content)




import os
from django import forms
from django.forms import ModelForm
from django.conf import settings

from apps.preferences.models import Preferences

# import re
# SHIELD_SYMBOLS = r'[#=!?*]'


class PreferencesForm(ModelForm):

    def __init__(self, *args, **kwargs):
        super(PreferencesForm, self).__init__(*args, **kwargs)
        if self.initial:
            self.fields['key'].label = ''
            self.fields['key'].widget.attrs = {'readonly': 'readonly', 'style': 'display: none;'}
            self.fields['description'].widget = forms.Textarea()
            self.fields['description'].widget.attrs = {'rows': 3, 'cols': 100, 'readonly': True,
                                                      'style': 'resize: none;'}

        if Preferences.KEY_REST_API in self.initial['key']:
            self.fields['value'].widget = forms.Select(choices=((True, 'True'), (False, 'False')))
        if Preferences.KEY_IMAGE_AUTOSCALE in self.initial['key']:
            self.fields['value'].widget = forms.Select(choices=((True, 'True'), (False, 'False')))

        if Preferences.KEY_MAX_INNERLINK_HISTORY in self.initial['key']:
            self.fields['value'].widget = forms.NumberInput()

        if Preferences.KEY_IMAGE_QUALITY in self.initial['key']:
            choices = tuple((str(i), str(i)) for i in range(100, 0, -10))
            self.fields['value'].widget = forms.Select(choices=choices)

        if Preferences.KEY_THEME in self.initial['key']:
            path_of_themes = os.path.join(settings.BASE_DIR, 'apps/pages/templates/themes/')
            choices = ((name, name) for name in os.listdir(path_of_themes)
                       if os.path.isdir(os.path.join(path_of_themes, name)))

            self.fields['value'].widget = forms.Select(choices=choices)


    class Meta:
        model = Preferences
        exclude = ['updated_at', 'created_at']

    #~ def clean_value(self):
        #~ new_value = self.cleaned_data['value']
        #~ match = re.search(SHIELD_SYMBOLS, new_value)
        #~ if match:
            #~ raise forms.ValidationError("Don\'t use symbols #=!?* ")
        #~ return new_value



"""
Django settings/production.py for MySmile deploy project.
"""
from .base import *
from config.production import *
from config.mysmile import *


APP_MIDDLEWARE_CLASSES = (
    'apps.api.middlewares.VersionSwitchMiddleware',
    'apps.utils.middlewares.ExceptionLoggingMiddleware',
    'apps.utils.middlewares.AdminLocaleOneLangMiddleware',
)


THIRD_PARTY_MIDDLEWARE_CLASSES = (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    ) if DEBUG==True else ()

MIDDLEWARE_CLASSES = DJANGO_MIDDLEWARE_CLASSES + \
                     APP_MIDDLEWARE_CLASSES + \
                     THIRD_PARTY_MIDDLEWARE_CLASSES
# Apps specific for this project go here.
LOCAL_APPS = (
    'apps.api',
    'apps.pages',
    'apps.preferences',
    'apps.sitemap',
    'apps.admin.update',
    'apps.admin.fail_login',
    'apps.utils',
    )

# another apps
THIRD_PARTY_APPS = ('debug_toolbar',
                    'compressor',
                    'rest_framework',
                    )

# See: https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS
# END APP CONFIGURATION

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'mysmile_cache_table',
    }
}

CACHE_MIDDLEWARE_SECONDS = 60*60*24

# compressor settings
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # other finders..
    'compressor.finders.CompressorFinder',
)

STATICFILES_DIRS = [os.path.join(BASE_DIR, 'mysmile/static/')]

COMPRESS_CSS_FILTERS = ['compressor.filters.css_default.CssAbsoluteFilter',  'compressor.filters.cssmin.CSSMinFilter']


if MYSMILE_ADMIN_FAIL_LOGIN_ENABLE:
    AUTHENTICATION_BACKENDS = ('apps.admin.fail_login.backends.FailLoginModelBackend',)
else:
    AUTHENTICATION_BACKENDS = ('django.contrib.auth.backends.ModelBackend',)


# production only
# python3 manage.py check --deploy
#SESSION_COOKIE_SECURE = True
SECURE_HSTS_SECONDS = 3600
X_FRAME_OPTIONS = 'DENY'
SECURE_HSTS_INCLUDE_SUBDOMAINS = True
SECURE_CONTENT_TYPE_NOSNIFF = True
SECURE_BROWSER_XSS_FILTER = True
#CSRF_COOKIE_SECURE = True
#CSRF_COOKIE_HTTPONLY = True
#SECURE_SSL_REDIRECT = True
#SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTOCOL", "https")



import os

from django.conf import settings
from django import template
register = template.Library()


@register.filter(name='emailprotect')
def emailprotect(email):
    filename = os.path.join(settings.STATIC_ROOT, 'themes/' + \
                            settings.MYSMILE_THEME + '/images/email2img.png')
    if os.path.exists(filename):
        return """<img src="/static/themes/""" + \
               settings.MYSMILE_THEME + \
               """/images/email2img.png" alt="email" />"""
    else:
        return '<a href="mailto:' + email + '">' + email + '</a>'



import os
import sys
# uncomment for use MySql
# import pymysql
# pymysql.install_as_MySQLdb()

if __name__ == "__main__":

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "mysmile.settings.local")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)





import datetime
from datetime import timedelta

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import PermissionDenied
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model
from django.contrib.auth.models import User
from django.core.mail import send_mail
from smtplib import SMTPRecipientsRefused

from apps.admin.fail_login.models import FailLogin

import logging
logger = logging.getLogger(__name__)  # Get an instance of a logger


class FailLoginModelBackend(ModelBackend):
    def authenticate(self, username=None, password=None, **kwargs):
        # fail_login injection to django authenticate
        count_fl = self.login_failed(username)
        if count_fl < settings.MYSMILE_ADMIN_FAIL_LOGIN_ATTEMPTS:
            # code below is django authenticate(self, username=None, password=None, **kwargs): ----
            UserModel = get_user_model()
            if username is None:
                username = kwargs.get(UserModel.USERNAME_FIELD)
            try:
                user = UserModel._default_manager.get_by_natural_key(username)
                if user.check_password(password):
                    return user
            except UserModel.DoesNotExist:
                # Run the default password hasher once to reduce the timing
                # difference between an existing and a non-existing user (#20760).
                UserModel().set_password(password)
            # -------------------------------------------------------------------------------------
        else:
            raise PermissionDenied  # 403 error

    def login_failed(self, username):
        try:
            user = User.objects.get(username=username)

            # delete records, older then MYSMILE_ADMIN_FAIL_LOGIN_TIMEOUT
            time = datetime.datetime.now() - timedelta(minutes=settings.MYSMILE_ADMIN_FAIL_LOGIN_TIMEOUT)
            FailLogin.objects.filter(created_at__lte=time).delete()

            count_fl = FailLogin.objects.filter(user=user).count()
            if count_fl < settings.MYSMILE_ADMIN_FAIL_LOGIN_ATTEMPTS:
                FailLogin(user=user).save()
                count_fl += 1
                # Send a one-time email to admin and user about temporary locked account
                if count_fl == settings.MYSMILE_ADMIN_FAIL_LOGIN_ATTEMPTS:
                    # try:
                    subject = '[MySmile] account temporary locked'
                    message = 'You account temporarily locked from '+time.strftime('%Y-%m-%d %H:%M:%S')+' to '+\
                              (time+timedelta(minutes=settings.MYSMILE_ADMIN_FAIL_LOGIN_TIMEOUT)).strftime('%Y-%m-%d %H:%M:%S')
                    try:
                        send_mail(subject, message, settings.SERVER_EMAIL, [settings.ADMINS[0][1]], fail_silently=False)
                    except SMTPRecipientsRefused as ex:
                        logger.error(ex)

                    user_email = User.objects.filter(username=username).values_list('email', flat=True)
                    try:
                        if user_email:
                            send_mail(subject, message, settings.SERVER_EMAIL, user_email, fail_silently=False)
                    except SMTPRecipientsRefused as ex:
                        logger.error(ex)

            else:
                raise PermissionDenied # 403 error
        except ObjectDoesNotExist: # username aren't exist
            logger.info('Attempt to login with username='+username)
        return count_fl


# MySmile settings
from apps.preferences.models import Preferences

MYSMILE_VERSION = '0.7.2'

try:
    MYSMILE_THEME = Preferences.objects.filter(key='THEME').values_list('value', flat=True)[0]
except Exception as err:
    MYSMILE_THEME = 'modern'

try:
    MYSMILE_REST_API = 'True' in Preferences.objects.filter(key='REST_API').values_list('value', flat=True)
except Exception as err:
    MYSMILE_REST_API = False

# Fail login settings
MYSMILE_ADMIN_FAIL_LOGIN_ENABLE = True
MYSMILE_ADMIN_FAIL_LOGIN_ATTEMPTS = 5
MYSMILE_ADMIN_FAIL_LOGIN_TIMEOUT = 15 # minutes



from django.http import HttpResponseNotFound
from django.template import RequestContext, loader, Template, TemplateDoesNotExist
from django.views.decorators.csrf import requires_csrf_token
from django.views.generic.base import RedirectView, TemplateView
from django.conf import settings
from django.core.cache import cache
from django.core import signing

import logging
logger = logging.getLogger(__name__)  # Get an instance of a logger

from apps.pages.models import Page
from apps.preferences.models import Preferences


class PageRedirectView(RedirectView):

    permanent = False
    query_string = True
    pattern_name = 'page'

    def get_redirect_url(self, *args, **kwargs):
        # get first slug
        slug = Page.objects.filter(status=Page.STATUS_PUBLISHED, ptype__in=[Page.PTYPE_MENU, Page.PTYPE_MENU_API]).values_list('slug', flat=True).order_by('sortorder').first()
        try:
            lang = kwargs['lang']
        except KeyError:  # adaptive language selection
            lang = settings.LANGUAGES[0][0]
            if 'HTTP_ACCEPT_LANGUAGE' in self.request.META:
                for k in settings.LANGUAGES:
                    if k[0] in self.request.META['HTTP_ACCEPT_LANGUAGE']:
                        lang = k[0]
                        k = 0  # break cycle "for"
        return super(PageRedirectView, self).get_redirect_url(lang=lang, slug=slug)


class PageView(TemplateView):

    def get_template_names(self):
        return settings.MYSMILE_THEME + '/page.html'

    def get_context_data(self, **kwargs):
        context = super(PageView, self).get_context_data(**kwargs)
        c = Page.objects.get_content(kwargs['lang'], kwargs['slug'])
        c.update(Preferences.objects.get_all())
        c['inav'] = self.get_additional_dynamic_menu(self.request, kwargs['slug'], c['menu'], c['page__ptype'], int(c['MAX_INNERLINK_HISTORY']))

        c['MYSMILE_THEME'] = settings.MYSMILE_THEME
        context.update(c)
        return context

    def get_additional_dynamic_menu(self, request, slug, menu, ptype, max_innerlink_history):
        inner_nav = request.session.get('inner_nav', [])
        if ptype == Page.PTYPE_INNER:
            if not [slug, menu] in inner_nav:  # work with sessions
                inner_nav.append([slug, menu]) # add to dynamic menu
                request.session['inner_nav'] = inner_nav  # save data to the session
                while len(inner_nav) > max_innerlink_history:
                    inner_nav.pop(0)
        return inner_nav





import json
import urllib

from django.contrib import messages
from django.views.generic.base import RedirectView
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

class CheckUpdate(RedirectView):
    permanent = False
    query_string = True

    def get_redirect_url(self, *args, **kwargs):
        try:
            url = 'https://api.github.com/repos/MySmile/mysmile/releases/latest'
            r = urllib.request.urlopen(url)
            latest_release = json.loads(r.read().decode(r.info().get_param('charset') or 'utf-8'))
            version = latest_release.get('tag_name')[1::]
            if version > settings.MYSMILE_VERSION:
                msg = _('New version ') + version + _(' is avaliable!') + '<a href="' +\
                      latest_release.get('zipball_url') + '">' + _(' download now') + '</a>'
            else:
                msg = _('You are up to date!')
        except Exeption as err:
            msg = str(err)

        messages.add_message(self.request, messages.INFO, msg, extra_tags='safe')
        return super(CheckUpdate, self).get_redirect_url(*args, **kwargs)



import datetime
from django.test import TestCase
from django.test.client import Client


class SitemapTestView(TestCase):
    fixtures = ['apps/pages/fixtures/pages.json']

    def setUp(self):
        self._client = Client()
        self.response = self._client.get('/Sitemap.xml', HTTP_HOST="testdomain.com")

    def test_sitemap(self):
        self.assertEqual(self.response.status_code, 200)

    def test_item_index(self):
        self.assertIn(b'index.html', self.response.content)



import pytz

from django.conf import settings

from rest_framework import serializers

from apps.preferences.models import Preferences
from apps.pages.models import Page, Page_translation

class DateTimeTzSerializer(serializers.Serializer):
    def to_native(self, value):
        localize = pytz.timezone(settings.TIME_ZONE).localize(value)
        return str(localize)

class ContactSerializer(serializers.Serializer):
    data =  Preferences.objects.get_contact()

class LangsSerializer(serializers.Serializer):
    data = [item[0] for item in settings.LANGUAGES]

class ContentSerializer(serializers.Serializer):
    menu = serializers.CharField()

#class ContentSerializer(serializers.ModelSerializer):
#    class Meta:
#        model = Page_translation
#        fields = ('menu', 'name', 'col_central')


    # content = Page_translation.objects.filter(lang=lang, page__status=Page.STATUS_PUBLISHED, page__ptype__in=[Page.PTYPE_API, Page.PTYPE_MENU_API]).order_by('page__sortorder').values_list('page__slug', 'menu')
    # data = [{item[0]: item[1]} for item in content]





from django.contrib import admin

from apps.preferences.models import Preferences
from apps.preferences.forms import PreferencesForm


class PreferencesAdmin(admin.ModelAdmin):
    model = Preferences
    form = PreferencesForm
    fieldsets = [
        ('Preferences', {'fields': ['value', 'description', 'key']}),
    ]
    list_display = ('name', 'value')
    list_display_links = ('name',)
    save_on_top = True
    actions = None

    def has_delete_permission(self, request, obj=None):
        """Disable 'delete' button
        """ 
        return False

    def has_add_permission(self, request, obj=None):
        """Disable 'add' button
        """ 
        return False

admin.site.register(Preferences, PreferencesAdmin)



from django.conf.urls import patterns, url
from rest_framework.urlpatterns import format_suffix_patterns
from django.views.decorators.csrf import csrf_exempt

from apps.api.v1.views import MySmileApi


urlpatterns = [
    url(r'(?P<resource>[a-z,0-9]+)$', csrf_exempt(MySmileApi.as_view())),
]

urlpatterns = format_suffix_patterns(urlpatterns)




import traceback
import sys

from django.conf import settings
from django.utils import translation

import logging
logger = logging.getLogger(__name__)


class ExceptionLoggingMiddleware(object):

    def process_exception(self, request, exception):
        logger.error('\n'.join(traceback.format_exception(*sys.exc_info())))


class AdminLocaleOneLangMiddleware:
    """ Autochange locale in admin if len(settings.LANGUAGES) == 1
    """
    def process_request(self, request):
        if len(settings.LANGUAGES) == 1 and request.path.startswith('/admin'):
            translation.activate(settings.LANGUAGES[0][0])
            request.LANGUAGE_CODE = settings.LANGUAGES[0][0]


from django.core.urlresolvers import resolve, reverse


class VersionSwitchMiddleware(object):

    def process_request(self, request):
        r = resolve(request.path_info)
        version = request.META.get('HTTP_X_MYSMILE_VERSION', False)
        if r.namespace.startswith('api:') and version:
            old_version = r.namespace.split(':')[-1]
            request.path_info = reverse('{}:{}'.format(r.namespace.replace(old_version, version), r.url_name), args=r.args, kwargs=r.kwargs)



import datetime
from django.test import TestCase
from django.test.client import Client

from apps.pages.models import Page, Page_translation


class ApiTestCase(TestCase):
    def setUp(self):
        some_page = Page.objects.create(id=1,
                        slug='index',
                        color='#FDA132',
                        photo='images/photo.png',
                        sortorder=1,
                        status=Page.STATUS_PUBLISHED,
                        ptype=Page.PTYPE_API,
                        updated_at=datetime.datetime.now(),
                        created_at=datetime.datetime.now())
        
        Page_translation.objects.create(id=1,
                        page=some_page,
                        lang='en',
                        menu='Main',
                        col_central='lorem ipsum',
                        col_bottom_1='lorem ipsum',
                        col_bottom_2='lorem ipsum',
                        col_bottom_3='lorem ipsum',
                        meta_title='Welcome!',
                        meta_description='This is mane page!',
                        meta_keywords='Python3, Django',
                        photo_alt='',
                        photo_description='',
                        updated_at=datetime.datetime.now(),
                        created_at=datetime.datetime.now())
        self._client = Client()

    def test_content_short(self):
        response = self._client.get('/api/v1/content')
        self.assertEqual(response.status_code, 200)

    def test_content_slug(self):
        response = self._client.get('/api/v1/content?slug=index')
        self.assertEqual(response.status_code, 200)

    def test_content_slug_lang(self):
        response = self._client.get('/api/v1/content?slug=index&lang=en')
        self.assertEqual(response.status_code, 200)

    def test_language(self):
        response = self._client.get('/api/v1/language')
        self.assertEqual(response.status_code, 200)

    def test_contact(self):
        response = self._client.get('/api/v1/contact')
        self.assertEqual(response.status_code, 200)



from django.conf import settings


def mysmile_version(request):
    return {'MYSMILE_VERSION': settings.MYSMILE_VERSION}

def mysmile_theme(request):
    return {'MYSMILE_THEME': settings.MYSMILE_THEME}



from django.conf.urls import patterns, url
from rest_framework.urlpatterns import format_suffix_patterns

from apps.api.v2.views import ContactView, LangsView, ContentView


urlpatterns = [
    url(r'contact/$', ContactView.as_view()),
    url(r'language/$', LangsView.as_view()),
    url(r'content', ContentView.as_view()),
]

urlpatterns = format_suffix_patterns(urlpatterns)


from django.conf.urls import patterns, url
from django.http import HttpResponse
from django.conf import settings

urlpatterns = patterns('',
    url(r'^Sitemap\.xml$', 'apps.sitemap.views.SitemapXML'),

    url(r'^robots\.txt$', lambda r: HttpResponse("User-agent: *\nHost: " + \
                                              settings.ALLOWED_HOSTS[0] +
                                              "\nSitemap: http://" + settings.ALLOWED_HOSTS[0] + \
                                              "/Sitemap.xml", content_type="text/plain")),
    #(r'^robots\.txt$', lambda r: HttpResponse("User-agent: *\nDisallow: /", mimetype="text/plain")),
)



from django import template
register = template.Library()

import logging
logger = logging.getLogger(__name__)


@register.filter(name='embed')
def embed(youtube_url):
    try:
        code = youtube_url.split('=')[-1]
        youtube_url = 'https://www.youtube.com/embed/' + code + '?feature=player_detailpage'
    except Exception as err:
        youtube_url = ''
        logger.error(str(err))
    return youtube_url    



from django.db import models
from django.utils.translation import ugettext_lazy as _


class PreferencesManager(models.Manager):
    def get_all(self):
        p = Preferences.objects.all().values('key', 'value')
        c = {}
        for item in p:
            c[item['key']] = item['value']
        return c

    def get_contact(self):
        p = Preferences.objects.filter(key__in=['PHONE', 'EMAIL', 'SKYPE']).values('key', 'value')
        c = {}
        for item in p:
            c[item['key'].lower()] = item['value']
        return c


class Preferences(models.Model):
    KEY_PHONE = 'PHONE'
    KEY_EMAIL = 'EMAIL'
    KEY_SKYPE = 'SKYPE'
    KEY_GOOGLE_ANALITYCS_CODE = 'GOOGLE_ANALITYCS_CODE'
    KEY_MAX_INNERLINK_HISTORY = 'MAX_INNERLINK_HISTORY'
    KEY_REST_API = 'REST_API'
    KEY_IMAGE_QUALITY = 'IMAGE_QUALITY'
    KEY_THEME = 'THEME'
    KEY_IMAGE_AUTOSCALE = 'IMAGE_AUTOSCALE'

    CONTACT = {KEY_PHONE: 'phone', KEY_EMAIL: 'email', KEY_SKYPE: 'skype'}

    DEFAULT = ({'key': KEY_PHONE, 'value': '000 000 000 00 00',
                'name': 'Phone number', 'description': 'Contact phone number. It is visible on the top of site page.'},
               {'key': KEY_EMAIL, 'value': 'myemail@email.com',
                'name': 'Email', 'description': 'Contact Email. It is visible on the top of site page as an image to protect from spam bot.'},
               {'key': KEY_SKYPE, 'value': 'myskype',
                'name': 'Skype', 'description': 'Contact skype. It is visible on the top of site page.'},
               {'key': KEY_GOOGLE_ANALITYCS_CODE, 'value': '',
                'name': 'Google Analytic code', 'description': 'Code uses to connect to Google Analytic service.'},
               {'key': KEY_MAX_INNERLINK_HISTORY, 'value': 4,
                'name': 'Max number of inner pages', 'description': 'Number of pages that can be shown under main menu when user follow links inside page. Generally inner pages used only as a link inside content.'},
               {'key': KEY_REST_API, 'value': True,
                'name': 'Turn on/off REST Api', 'description': 'Turn on/off alternative way of getting pages using REST api. Only a special marked pages can be available for api. For more information please look into page statuses list.'},
               {'key': KEY_IMAGE_QUALITY, 'value': 100,
                'name': 'Image quality', 'description': 'Global setting quality uploaded pictures. The picture quality is directly proportional to the size of the file. A value of 100 corresponds to the maximum quality.'},
               {'key': KEY_THEME, 'value': 'modern',
                'name': 'Theme switcher', 'description': 'Choose a theme for switch'},
               {'key': KEY_IMAGE_AUTOSCALE, 'value': True,
                'name': 'Image autoscale', 'description': 'Global setting for autoscale uploaded pictures if its width>333px. Pictures will be scaled to width=333px.'},)

    key = models.CharField(unique=True, max_length=255, verbose_name=_('key'))
    value = models.CharField(blank=True, null=True, max_length=255, verbose_name=_('value'))
    name = models.CharField(max_length=255, verbose_name=_('name'))
    description = models.CharField(max_length=255, verbose_name=_('description'))
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    objects = PreferencesManager()

    def __setattr__(self, name, value):
        if name.isupper():
            raise AttributeError(name + " is an immutable attribute.")
        else:
            self.__dict__[name] = value

    class Meta:
        db_table = 'Preferences'
        ordering = ['name']
        verbose_name = _('Preference')
        verbose_name_plural = _('Preferences')




"""
Django settings for test1 project.

Generated by 'django-admin startproject' using Django 1.8.5.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import logging
logging.disable(logging.CRITICAL)

BASE_DIR = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), '..')


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'nj#3fo$7waf0@pj%w+%t^jumu#%fr37ex#9#i8kg&)sq)2rf-f'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

COMPRESS_ENABLED = False

ALLOWED_HOSTS = ['127.0.0.1', 'localhost']


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'apps.api',
    'apps.pages',
    'apps.preferences',
    'apps.sitemap',
    'apps.admin.update',
    'apps.admin.fail_login',
    'apps.utils',

    'compressor',
    'rest_framework',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',

    'apps.api.middlewares.VersionSwitchMiddleware',
    'apps.utils.middlewares.ExceptionLoggingMiddleware',
    'apps.utils.middlewares.AdminLocaleOneLangMiddleware',
)

ROOT_URLCONF = 'mysmile.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'mysmile/templates'),
            os.path.join(BASE_DIR, 'apps/pages/templates/themes'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                 # Insert your TEMPLATE_CONTEXT_PROCESSORS here
                'apps.utils.context_processors.mysmile_version',
                'apps.utils.context_processors.mysmile_theme',
                 # or use this list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
            'loaders': [
                ('django.template.loaders.cached.Loader', [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
                ]),
            ],
        },
    },
]

WSGI_APPLICATION = 'mysmile.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': '',
        'TEST': {
            'NAME': os.path.join(BASE_DIR, 'mysmile/db/test_mysmile.sqlite3'),
        },
    }
}


# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

STATIC_URL = '/static/'

# Enter path to media folder
MEDIA_ROOT = os.path.join(BASE_DIR, 'mysmile/media/')

# Enter path to static folder
STATIC_ROOT = os.path.join(BASE_DIR, 'mysmile/static/')

COMPRESS_ROOT = STATIC_ROOT

USE_ETAGS = True

REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.AllowAny'
    ],
    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.NamespaceVersioning'
}

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

SESSION_ENGINE = 'django.contrib.sessions.backends.cached_db'

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'mysmile_cache_table',
        # 'TIMEOUT': None,
    }
}

CACHE_MIDDLEWARE_SECONDS = 5 #60*60*24

# compressor settings
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # other finders..
    'compressor.finders.CompressorFinder',
)

COMPRESS_CSS_FILTERS = ['compressor.filters.css_default.CssAbsoluteFilter',  'compressor.filters.cssmin.CSSMinFilter']




from django.shortcuts import render
from django.http import HttpResponseNotFound
from django.template import RequestContext, loader, Template, TemplateDoesNotExist
from django.views.decorators.csrf import requires_csrf_token

import logging
logger = logging.getLogger(__name__)  # Get an instance of a logger

from apps.pages.models import Page, Page_translation


@requires_csrf_token
def e404(request, template_name='404.html'):
    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        template = Template(
            '<h1>Not Found</h1>'
            '<p>The requested URL {{ request_path }} was not found on this server.</p>')
    try:
        slug = request.path.split('/')[-1].split('.html')[0]  # get slug from path request
        #  Verify the existence of the slug
        slug = Page.objects.filter(slug=slug,
                                   status=Page.STATUS_PUBLISHED,
                                   ptype__in=[Page.PTYPE_MENU, Page.PTYPE_MENU_API, Page.PTYPE_INNER]).values_list('slug', flat=True)[0]
    except IndexError as err:
        slug = None
        logger.error(str(err))

    langs = Page_translation.objects.filter(page__slug=slug).values_list('lang', flat=True) if slug else ''
    return HttpResponseNotFound(template.render(RequestContext(request, {'request_host': request.get_host, 'request_path': request.path, 'slug': slug, 'langs': langs})))


@requires_csrf_token
def e500(request, template_name='500.html'):
    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        template = Template(
            '<h1>Not Found</h1>'
            '<p>The requested URL {{ request_path }} was not found on this server.</p>')
    return HttpResponseNotFound(template.render(RequestContext(request, {'request_path': request.path, })))

# like  e403
def csrf_failure(request, reason=""):
    template_name = '403.html'
    try:
        template = loader.get_template(template_name)
    except TemplateDoesNotExist:
        template = Template(
            '<b>HTTP Forbidden</b>'
            '<p>The requested URL {{ request_path }} forbidden.</p>')
    logger.error('error 403: ' + str(request))
    return HttpResponseNotFound(template.render(RequestContext(request, {'request_path': request.path, })))





from django.http import HttpResponse

from apps.pages.models import Page_translation, Page


def createNode(url, modified):
    return '<url><loc>' + url + '</loc><changefreq>weekly</changefreq>\
<lastmod>' + modified + '</lastmod></url>'


def SitemapXML(request):
    xml = '<?xml version = "1.0" encoding = "UTF-8"?>\
    <urlset xmlns = "http://www.sitemaps.org/schemas/sitemap/0.9">'
    langs_and_slugs = Page_translation.objects.filter(page__status=Page.STATUS_PUBLISHED, page__ptype__in=[Page.PTYPE_INNER, Page.PTYPE_MENU, Page.PTYPE_MENU_API]).values('lang', 'page__slug', 'updated_at').order_by('lang')
    for item in langs_and_slugs:
        url = 'http://' + request.META['HTTP_HOST'] + '/' + item['lang'] + '/' + item['page__slug'] + '.html'
        modified = item['updated_at'].strftime('%Y-%m-%d')
        xml += createNode(url, modified)
    xml += '</urlset>'
    return HttpResponse(xml, content_type="text/xml")



from django.test import TestCase

from django.utils import timezone

from apps.pages.models import Page


class PageTestCase(TestCase):
    fixtures = ['pages.json']

    def setUp(self):
        pass

    def test_page_field_created_at(self):
        pass
        # p = Page.objects.create(
        # slug='slug',
        # color='#FDA132',
        # photo='images/photo.png',
        # status=Page.STATUS_PUBLISHED,
        # ptype=Page.PTYPE_MENU)
        # p.save()
        # # local = timezone.localtime(timezone.now())
        # local = timezone.now()
        # created_at = p.created_at
        # delta = (local-created_at).total_seconds() # in seconds
        # self.assertTrue(delta<5)






import os
from io import BytesIO
from PIL import Image, ImageDraw, ImageFont

from django.test import TestCase
from django.conf import settings
from apps.preferences.models import PreferencesManager


class SignalsTestCase(TestCase):
    fixtures = ['preferences.json']

    def setUp(self):
        pass

    def test_email2img(self):
        p = PreferencesManager().get_all()
        email = p.get('EMAIL')
        color_mode = "RGBA"
        background_color = (0, 0, 0, 0)  # full transparent
        fontfile = os.path.join(settings.STATIC_ROOT, 'fonts/TimesNewRomanCE.ttf')
        fontsize = 16
        textcolor = (119 , 119, 119)

        font = ImageFont.truetype(fontfile, fontsize)
        width, height = font.getsize(email)

        file = BytesIO()
        image = Image.new(color_mode, (width, height + fontsize % 10), background_color)
        draw = ImageDraw.Draw(image)
        draw.text((0, 0), email, textcolor, font=font)
        image.save(file, 'png')
        file.name = 'testemail.png'
        file.seek(0)

        self.assertTrue(isinstance(file.read(), bytes))

    def test_clear_photo_file(self):
        pass




from django.conf.urls import patterns, url

from apps.pages.views import PageView, PageRedirectView


urlpatterns = patterns('',
    url(r'^$', PageRedirectView.as_view(), name='home'),
    url(r'^(?P<lang>[a-z]{2})/$', PageRedirectView.as_view(), name='page'),
    url(r'^(?P<lang>[a-z]{2})/(?P<slug>[a-zA-Z0-9-]+)\.html$', PageView.as_view(), name='page'),
    )



from django.db import models
from django.contrib.auth.signals import user_logged_in, user_login_failed
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _


class FailLogin(models.Model):
    """ Protect admin login from brute-force attack
    """
    user = models.ForeignKey(User, editable = False, verbose_name=_('User'))
    created_at = models.DateTimeField(auto_now_add=True)#(default=datetime.datetime.now())#

    def __unicode__(self):
        return self.user

    class Meta:
        db_table = 'FailLogin'
        verbose_name = _('Fail login')
        verbose_name_plural = _('Fail logins')


def fl_logged_in(sender, user, request, **kwargs):
    # When a user successfully logs in, delete all "failed_logins" entries for that user
    FailLogin.objects.filter(user=user).delete()

user_logged_in.connect(fl_logged_in)



import time

from django.conf import settings
from rest_framework.views import exception_handler


def custom_exception_handler(exc, context):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    response = exception_handler(exc, context)

    # Now add the HTTP status code to the response.
    if response is not None:
        response.data = {'results': {'code': response.status_code, 'msg': response.data['detail']}}

    return response

#
# class MySmileApiException(Exception):
#
#     def __init__(self, msg, code):
#         self.msg = msg
#         self.code = code



import os
from django.contrib import admin
from django.conf import settings

from apps.pages.models import Page, Page_translation
from apps.pages.forms import Page_translationInlineForm, PageForm


class Page_translationInline(admin.StackedInline):
    model = Page_translation
    form = Page_translationInlineForm
    verbose_name = 'Lang'
    extra = 0
    fieldsets = [
        ('Content', {'fields': ['lang', 'menu', 'name', 'col_central',
                                'youtube', 'photo_description', 'col_right',
                                'col_bottom_1', 'col_bottom_2', 'col_bottom_3'],
                    'classes': ['collapse']}),
        ('SEO', {'fields': ['meta_title', 'meta_description', 'meta_keywords',
        'photo_alt'], 'classes': ['collapse']}),
    ]
    search_fields = ['col_central', 'col_right', 'col_bottom_1', 'col_bottom_2',
                     'col_bottom_3']
    max_num = len(settings.LANGUAGES)

    def get_queryset(self, request):
        return Page_translation.objects.filter(lang__in=[x[0] for x in settings.LANGUAGES])

class PageAdmin(admin.ModelAdmin):
    form = PageForm
    inlines = [Page_translationInline]
    save_on_top = True
    readonly_fields = ('photo_thumb',)
    view_on_site = True

    def date_update(self, model):
        return model.updated_at.strftime('%d %B %Y, %H:%M')

    def waiting_for_translation(self, model):
        """ Flag doesn't display if translation prepared
        """
        flags = ''
        for item in settings.LANGUAGES:
            if not Page_translation.objects.filter(page_id=model.id, lang=item[0]):
                flags += """<img src="/static/themes/""" + settings.MYSMILE_THEME + \
                         """/images/flags/""" + item[0] + """.png" alt= " """ + \
                         item[1] + """ "/>"""
        return flags
    waiting_for_translation.short_description = 'waiting for translation'
    waiting_for_translation.allow_tags = True


    def get_list_display(self, request):
        """
        Hide empty colums "photo_thumb" and "waiting_for_translation"
        """
        pages = Page.objects.all().count()
        pages_translation = Page_translation.objects.all().count()
        pages_blankphoto = Page.objects.filter(photo='').count()

        self.list_display = ('slug', 'status', 'ptype', 'sortorder',)

        if pages_blankphoto < pages:  # at least one photo exist
            self.list_display += ('photo_thumb', )
        if pages*len(settings.LANGUAGES) != pages_translation:
            self.list_display += ('waiting_for_translation',)

        return self.list_display + ('date_update',)

    def get_fieldsets(self, request, obj=None):
        fieldsets = super(PageAdmin, self).get_fieldsets(request, obj)
        if obj:
            photo = Page.objects.filter(id=obj.id).values_list('photo', flat=True)[0]
            if photo:
                fieldsets = [('Settings', {'fields': ['slug', 'status', 'ptype', 'sortorder',
                                                      'color', ('photo', 'photo_thumb')]}), ]
            else:
                fieldsets = [('Settings', {'fields': ['slug', 'status', 'ptype', 'sortorder',
                                                      'color', ('photo',)]}), ]

        return fieldsets


admin.site.register(Page, PageAdmin)




import json
from django.http import HttpResponse
from django.views.generic.base import View
from django.db import DatabaseError
from django.core.exceptions import FieldError
from django.conf import settings

from apps.pages.models import Page, Page_translation
from apps.api.v1.exceptions import MySmileApiException
from apps.preferences.models import Preferences


import logging
logger = logging.getLogger(__name__)


class MySmileApi(View):

    def dispatch(self, request, *args, **kwargs):
        api_on_off = Preferences.objects.filter(key='REST_API').values('value')
        if 'False' == api_on_off:
            raise MySmileApiException('Forbidden', 403)
        return super(MySmileApi, self).dispatch(request, *args, **kwargs)

    def get(self, request, resource):
        self.lang = request.GET.get('lang', 'en')
        self.slug = request.GET.get('slug', '')

        try:
            if resource == 'content':
                response_data = self.get_content(request)
            elif resource == 'language':
                response_data = self.get_language()
            elif resource == 'contact':
                response_data = self.get_contact()
            else:
                raise MySmileApiException('Not Found', 404)
        except IndexError:
            response_data = {'code': 404, 'msg': 'Not Found'}

        except MySmileApiException as inst:
            response_data = {'code': inst.code, 'msg': inst.msg}

        except (DatabaseError, FieldError, KeyError, Exception) as err:
            logger.error(err)
            # @FIXME: save exception details to log
            response_data = {'code': 500, 'msg': 'Internal Server Error'}

        return HttpResponse(json.dumps(response_data), content_type="application/json",
                            status=response_data['code'])

    def get_content(self, request):
        response_data = {'code': 200, 'data': {}}

        if self.slug == '':
            # get list of pages
            content = Page_translation.objects.filter(lang=self.lang, page__status=Page.STATUS_PUBLISHED, page__ptype__in=[Page.PTYPE_API, Page.PTYPE_MENU_API]).order_by('page__sortorder').values_list('page__slug', 'menu')

            if not content:
                raise MySmileApiException('Not Found', 404)
            response_data['data'] = [{item[0]: item[1]} for item in content]

            return response_data

        # get current page by slug
        page_id = Page.objects.filter(slug=self.slug, status=Page.STATUS_PUBLISHED, ptype__in=[Page.PTYPE_API, Page.PTYPE_MENU_API]).values('id')
        if not page_id:
            raise MySmileApiException('Not Found', 404)

        content = Page_translation.objects.filter(lang=self.lang, page__status=Page.STATUS_PUBLISHED, page_id=page_id, page__ptype__in=[Page.PTYPE_API, Page.PTYPE_MENU_API]).values(
            'page__color', 'page__photo', 'menu', 'name',
            'col_central', 'col_right', 'youtube', 'col_bottom_1',
            'col_bottom_2', 'col_bottom_3', 'photo_alt', 'photo_description',
            'meta_title', 'meta_description', 'meta_keywords')[0]

        if not content:
            raise MySmileApiException('Not Found', 404)

        response_data['data'] = {'menu': content['menu'],
                                 'name': content['name'],
                                 'col_central': content['col_central'],
                                 'col_right': content['col_right'],
                                 'youtube': content['youtube']
                                }

        response_data['data']['col_bottom'] = [content[item] for item in ['col_bottom_1', 'col_bottom_2', 'col_bottom_3'] if content[item]]

        response_data['data']['photo'] = {'src': request.build_absolute_uri('/static/' + content['page__photo']),
                                          'alt': content['photo_alt'],
                                          'description': content['photo_description']
                                          }

        return response_data

    def get_contact(self):
        response_data = {'code': 200}
        response_data['data'] = Preferences.objects.get_contact()
        return response_data

    def get_language(self):
        response_data = {'code': 200}
        response_data['data'] = [item[0] for item in settings.LANGUAGES]

        return response_data

    def post(self, request, resource):
        response_data = {'code': 502, 'msg': 'Method Not Allowed'}

        return HttpResponse(json.dumps(response_data), content_type="application/json", status=502)

    def put(self, request, resource):
        response_data = {'code': 502, 'msg': 'Method Not Allowed'}

        return HttpResponse(json.dumps(response_data), content_type="application/json", status=502)

    def delete(self, request, resource):
        response_data = {'code': 502, 'msg': 'Method Not Allowed'}

        return HttpResponse(json.dumps(response_data), content_type="application/json", status=502)



from django.conf.urls import patterns, include, url, handler404
from django.conf import settings
from django.contrib.staticfiles.views import serve as serve_static
from django.views.decorators.cache import never_cache
from django.contrib import admin


urlpatterns = patterns('',
    url('', include('apps.pages.urls')),
    url(r'^api/', include('apps.api.urls')),

    url('', include('apps.sitemap.urls')),

    url('', include('apps.admin.urls')),
    url(r'^api/', include('apps.api.urls', namespace='api')),

    url(r'^admin/', include(admin.site.urls)),
    (r'^i18n/', include('django.conf.urls.i18n')),
)


handler404 = 'apps.preferences.views.e404'
handler500 = 'apps.preferences.views.e500'

admin.site.site_header = 'MySmile administration'

if settings.DEBUG:
    urlpatterns += patterns('django.views.static',
        (r'media/(?P<path>.*)', 'serve', {'document_root': settings.MEDIA_ROOT}),
        url(r'^static/(?P<path>.*)$', never_cache(serve_static)),
    )

    import debug_toolbar
    urlpatterns += patterns('',
        url(r'^__debug__/', include(debug_toolbar.urls)),
    )






from django.test import TestCase
from django.test.client import Client

from apps.preferences.models import PreferencesManager


class PreferencesTestCase(TestCase):
    fixtures = ['preferences.json']

    def setUp(self):
        pass

    def test_api_key_true(self):
        p = PreferencesManager().get_all()
        api_key = p.get('REST_API')
        self.assertEqual(api_key, 'True')

    def test_keys_true(self):
        p_keys = PreferencesManager().get_all().keys()

        self.assertTrue(len(p_keys) == 9)
        self.assertTrue('PHONE' in p_keys)
        self.assertTrue('EMAIL' in p_keys)
        self.assertTrue('SKYPE' in p_keys)
        self.assertTrue('GOOGLE_ANALITYCS_CODE' in p_keys)
        self.assertTrue('MAX_INNERLINK_HISTORY' in p_keys)
        self.assertTrue('REST_API' in p_keys)
        self.assertTrue('IMAGE_QUALITY' in p_keys)
        self.assertTrue('IMAGE_AUTOSCALE' in p_keys)
        self.assertTrue('THEME' in p_keys)

    def test_REST_API_switch(self):
        pass



from django.conf.urls import url, patterns, include

urlpatterns = patterns('',
    url(r'', include('apps.api.v1.urls', namespace='default')),
    # url(r'^v2/', include('apps.api.v2.urls', namespace='v2')),
)




import os
from PIL import Image, ImageDraw, ImageFont

from django.db import connections, transaction
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.conf import settings
from django.contrib.admin.models import LogEntry

from apps.pages.models import Page
from apps.preferences.models import Preferences

import logging
logger = logging.getLogger(__name__)


@receiver(post_save)
def clear_cache(sender, instance=None, created=False, **kwargs):
    list_of_models = ('Page', 'Page_translation', 'Preferences')
    if sender.__name__ in list_of_models:
        cache_table = settings.CACHES['default']['LOCATION']
        try:
            with transaction.atomic():
                cursor = connections['default'].cursor()
                cursor.execute(' '.join(['DELETE FROM ', cache_table]))
        except Exception as err:
            logger.error(err)

    if sender.__name__=='Preferences':
        try:
            settings.MYSMILE_THEME = Preferences.objects.filter(key='THEME').values_list('value', flat=True)[0]
        except (IndexError, Exception) as err:
            settings.MYSMILE_THEME = 'modern'
            logger.error(err)

        try:
            settings.MYSMILE_REST_API = 'True' in Preferences.objects.filter(key='REST_API').values_list('value', flat=True)
        except Exception as err:
            settings.MYSMILE_REST_API = False
            logger.error(err)


def clear_photo_file(sender, instance, **kwargs):
    file = getattr(instance, 'photo')
    if file and os.path.exists(file.path):
        os.remove(file.path)


def email2img(sender, instance, created=False, **kwargs):
    """ protect email via image
    """

    if created == False and not isinstance(sender, LogEntry):
        email = Preferences.objects.filter(key='EMAIL').values_list('value', flat=True)
        if email:
            color_mode = "RGBA"
            background_color = (0, 0, 0, 0)  # full transparent
            fontfile = os.path.join(settings.STATIC_ROOT, 'fonts/TimesNewRomanCE.ttf')
            fontsize = 16
            textcolor = (119, 119, 119)
            try:
                font = ImageFont.truetype(fontfile, fontsize)
                width, height = font.getsize(email[0])
                # add fontsize%10 for fix some visual bug
                im = Image.new(color_mode, (width, height + fontsize % 10), background_color)
                draw = ImageDraw.Draw(im)
                draw.text((0, 0), email[0], textcolor, font=font)
                img_full_path = settings.STATIC_ROOT + 'themes/'+ \
                                settings.MYSMILE_THEME +'/images/email2img.png'
                im.save(img_full_path)
            except Exception as err:
                logger.error(err)


def image_tuning(sender, instance, created=False, **kwargs):
    """ Implement preferences: IMAGE_QUALITY and IMAGE_AUTOSCALE
    """
    try:
        path = instance.photo.path
        if os.path.isfile(path):
            quality = int(Preferences.objects.filter(key='IMAGE_QUALITY').values_list('value', flat=True)[0])
            image = Image.open(path)
            image.save(path, quality=quality, optimize=True)

        autoscale = Preferences.objects.filter(key='IMAGE_AUTOSCALE').values_list('value', flat=True)[0]
        if os.path.isfile(path) and autoscale:
            image = Image.open(path)
            if image.width > 333: # 333px in right. TODO: create constan like MYSMILE_IMAGE_WIDTH = 333
                wpercent = (333/float(image.size[0]))
                hsize = int((float(image.size[1])*float(wpercent)))
                image = image.resize((333, hsize), PIL.Image.ANTIALIAS)
                image.save(path)
    except Exception as err:
        logger.error(err)

# group by sender
pre_delete.connect(clear_photo_file, sender=Page, dispatch_uid="clear_photo_file")
post_save.connect(image_tuning, sender=Page, dispatch_uid="image_tuning")

post_save.connect(email2img, sender=Preferences, dispatch_uid="email2img")



from django.db import models
from django.http import Http404
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

import logging
logger = logging.getLogger(__name__)  # Get an instance of a logger

from django.conf import settings


class ImageField(models.ImageField):

    def save_form_data(self, instance, data):
        if data is not None:
            file = getattr(instance, self.attname)
            if file != data:
                file.delete(save=False)
        super(ImageField, self).save_form_data(instance, data)

def create_default_sortorder():
    default_sortorder = Page.objects.all().aggregate(models.Max('sortorder'))['sortorder__max']
    default_sortorder = default_sortorder + Page.SORTORDER_STEP if default_sortorder else Page.SORTORDER_STEP
    return default_sortorder

class PagesManager(models.Manager):
    def get_content(self, lang=None, slug=None):
        c = self.get_page(lang, slug)
        c['main_menu'] = list(self.get_main_menu(lang))
        return c

    def get_main_menu(self, lang):
        main_menu = Page_translation.objects.filter(lang=lang, page__status=Page.STATUS_PUBLISHED, page__ptype__in=[Page.PTYPE_MENU,Page.PTYPE_MENU_API]).values('page__slug', 'menu').order_by('page__sortorder')
        return main_menu

    def get_page(self, lang, slug):
        try:
            page = Page_translation.objects.filter(lang=lang, page__ptype__in=[Page.PTYPE_INNER, Page.PTYPE_MENU, Page.PTYPE_MENU_API], page__status=Page.STATUS_PUBLISHED, page__slug=slug).values('page__color', 'page__photo', 'menu', 'name', 'col_central', 'col_right', 'youtube', 'col_bottom_1', 'col_bottom_2', 'col_bottom_3', 'photo_alt', 'photo_description', 'meta_title', 'meta_description', 'meta_keywords', 'page__ptype')
        except Exception as err:
            logger.error(err)
            raise Http404

        if page:
            page[0]['bottom_cols'] = list(filter(None, [page[0]['col_bottom_1'], page[0]['col_bottom_2'], page[0]['col_bottom_3']]))
            return page[0]
        else:
            raise Http404


class Page(models.Model):
    PTYPE_INNER = 0
    PTYPE_MENU = 1
    PTYPE_API = 2
    PTYPE_MENU_API = 3
    PTYPE = ((PTYPE_INNER, _('inner page')),  # website only
             (PTYPE_MENU, _('menu page')),  # website only
             (PTYPE_API, _('API page')),   # API only
             (PTYPE_MENU_API, _('API & menu page')))  # API & website

    STATUS_DRAFT = 0
    STATUS_PUBLISHED = 1
    STATUS = ((STATUS_DRAFT, _('draft')),
              (STATUS_PUBLISHED, _('published')))
    SORTORDER_STEP = 10

    slug = models.SlugField(unique=True, verbose_name=_('slug'))
    color = models.CharField(max_length=255, default='#FDA132',
                             help_text=_('Click once with the mouse to select \
                                        a color, and then twice to save. TEMPORARY DISABLED IN NON-CLASSIC THEMES!'),
                             verbose_name=_('color'))
    # blank=True add "clear image" checkbox into admin page
    photo = ImageField(upload_to='images/', null=True, blank=True, verbose_name=_('photo'))
    sortorder = models.IntegerField(unique=True, default=create_default_sortorder, verbose_name=_('Sort order'))
    status = models.IntegerField(unique=False, choices=STATUS, default=STATUS_DRAFT, verbose_name=_('status'))
    ptype = models.IntegerField(unique=False, choices=PTYPE, default=PTYPE_MENU, verbose_name=_('page type'))
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = PagesManager()

    def photo_thumb(self):
        if self.photo:
            try:
                description = ''.join(['<img src="', self.photo.url, '" height="32"/> ',
                                     str(round(self.photo.size/1024, 2)), 'K, '
                                    'WxH: ', str(self.photo.width), 'x',
                                    str(self.photo.height), 'px'])
            except FileNotFoundError:
                return ''
            return description
        else:
            return ''
    photo_thumb.allow_tags = True

    def __setattr__(self, name, value):
        if name.isupper():
            raise AttributeError(name + " is an immutable attribute.")
        else:
            self.__dict__[name] = value
            
    def __unicode__(self):
        return self.slug

    class Meta:
        db_table = 'Page'
        ordering = ['-ptype', 'sortorder', 'status']
        verbose_name = _('Page')
        verbose_name_plural = _('Pages')

class Page_translation(models.Model):
    page = models.ForeignKey(Page)
    lang = models.CharField(max_length=255, choices=settings.LANGUAGES,
                            default=settings.LANGUAGES[0][0], verbose_name=_('lang'))
    menu = models.CharField(max_length=255, verbose_name=_('menu'))
    name = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('name'))
    col_central = models.TextField(blank=False, null=False, verbose_name=_('central column'))
    youtube = models.CharField(max_length=255, blank=True, null=True,
                               help_text=_('Link to youtube video. \
                               Max length url =  2048 characters'), verbose_name=_('youtube'))
    col_right = models.TextField(blank=True, null=True, verbose_name=_('right column'))
    col_bottom_1 = models.TextField(blank=True, null=True, verbose_name=_('bottom column 1'))
    col_bottom_2 = models.TextField(blank=True, null=True, verbose_name=_('bottom column 2'))
    col_bottom_3 = models.TextField(blank=True, null=True, verbose_name=_('bottom column 3'))

    meta_title = models.CharField(max_length=255, verbose_name=_('meta title'))
    meta_description = models.CharField(max_length=255, verbose_name=_('meta description'))
    meta_keywords = models.CharField(max_length=255, verbose_name=_('meta keywords'))
    photo_alt = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('photo alt'))
    photo_description = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('photo description'))

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
 
    def __unicode__(self):
        return self.lang

    class Meta:
        db_table = 'Page_translation'
        ordering = ['lang']
        verbose_name = _('Translation')
        verbose_name_plural = _('Translations')
        unique_together = ('page', 'lang')

    def get_absolute_url(self):
        return reverse('page', kwargs={'lang': self.lang, 'slug': self.page.slug})



# -*- coding: utf-8 -*-
"""
Django settings/base.py for MySmile project.
"""
import os, sys
import datetime

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), '..')


# APP CONFIGURATION
DJANGO_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)

DJANGO_MIDDLEWARE_CLASSES = (
    'django.middleware.common.BrokenLinkEmailsMiddleware',
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

ROOT_URLCONF = 'mysmile.urls'

WSGI_APPLICATION = 'mysmile.wsgi.application'

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = 'en'

USE_I18N = True

USE_L10N = True

USE_TZ = True
TIME_ZONE = 'UTC'

LANGUAGES = (
    ('uk', 'Українська'),
    ('en', 'English'),
#    ('pl', 'Polski'),
)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = '/static/'
MEDIA_URL = '/media/'

CSRF_FAILURE_VIEW = 'apps.preferences.views.csrf_failure'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'mysmile/templates'),
            os.path.join(BASE_DIR, 'apps/pages/templates/themes'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here
                'apps.utils.context_processors.mysmile_version',
                'apps.utils.context_processors.mysmile_theme',
                # or use this list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',

                # 'django.template.context_processors.debug',
                # 'django.template.context_processors.request',
                # 'django.contrib.auth.context_processors.auth',
                # 'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.AllowAny'
    ],
    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.NamespaceVersioning'
}

LOGGING = {
    'version': 1,
    # True if the default configuration is completely overridden
    'disable_existing_loggers': False,
    'formatters': {
         'verbose': {
             'format' :
                 '%(asctime)s: %(name)s, line %(lineno)d: '
                 '%(levelname)s: %(message)s ',
             'datefmt' : "%d/%b/%Y %H:%M:%S"
         },
         'simple': {
             'format': '%(levelname)s %(message)s'
         },
     },
    'filters': {
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue'
        },
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        },

    },
    'handlers': {
        'file': {
               'level': 'DEBUG',
               'class': 'logging.handlers.RotatingFileHandler',
               'formatter': 'verbose',
               'filename': os.path.join(BASE_DIR,  'log/'+datetime.datetime.now().strftime('%Y-%m-%d')+'.log'),
               'maxBytes': 1024*1024*5, # 5Mb
               'backupCount': 5
           },
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler',
            'filters': ['require_debug_false']
        },
        'console': {
            'level': 'DEBUG',
            'filters': ['require_debug_true'],
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
        'null': {
            "class": 'django.utils.log.NullHandler',
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins', 'console'],
            'level': 'ERROR',
            'propagate': True,
        },
        'django': {
            'handlers': ['null', ],
        },
        'django.db.backends': {
            'handlers': ['null', ],
            'propagate': False,
        },
        'py.warnings': {
            'handlers': ['null', ],
        },
        '': {
            'handlers': ['console', 'file'],
            'level': "DEBUG",
        },

    },
}

USE_ETAGS = True

LOCALE_PATHS = (os.path.join(BASE_DIR, 'mysmile/locale/'),
                )
